@@ -41,12 +41,133 @@ An abstract VHDL language model.
 :license: Apache License, Version 2.0
 """
 from enum     import IntEnum, unique, Enum
-from typing import List, Iterable
+from typing import List, Iterable, Union, Optional as Nullable, Dict
 
 from pydecor  import export
 
 
-__version__ = "0.11.4"
+__version__ = "0.13.0"
+
+SimpleOrAttribute =     Union['SimpleName',    'AttributeName']
+
+LibraryOrSymbol =       Union['Library',       'LibrarySymbol']
+EntityOrSymbol =        Union['Entity',        'EntitySymbol']
+ArchitectureOrSymbol =  Union['Architecture',  'ArchitectureSymbol']
+PackageOrSymbol =       Union['Package',       'PackageSymbol']
+ConfigurationOrSymbol = Union['Configuration', 'ConfigurationSymbol']
+ContextOrSymbol =       Union['Context',       'ContextSymbol']
+
+SubtypeOrSymbol =       Union['Subtype',       'SubtypeSymbol']
+
+ConstantOrSymbol =      Union['Constant',      'ConstantSymbol']
+VariableOrSymbol =      Union['Variable',      'VariableSymbol']
+SignalOrSymbol =        Union['Signal',        'SignalSymbol']
+
+ConstraintUnion = Union[
+	'RangeExpression',
+	'RangeAttribute',
+	'RangeSubtype',
+]
+
+ExpressionUnion = Union[
+	'BaseExpression',
+	'QualifiedExpression',
+	'FunctionCall',
+	'TypeConversion',
+	ConstantOrSymbol,
+	VariableOrSymbol,
+	SignalOrSymbol,
+	'Literal',
+]
+
+ContextUnion = Union[
+	'LibraryClause'
+	'UseClause'
+	'ContextReference'
+]
+
+
+@export
+@unique
+class VHDLVersion(Enum):
+	"""
+	An enumeration for all possible version numbers for VHDL.
+
+	A version can be given as integer or string and is represented as a unified
+	enumeration value.
+
+	This enumeration supports compare operators.
+	"""
+
+	VHDL87 =             87
+	VHDL93 =             93
+	VHDL2002 =         2002
+	VHDL2008 =         2008
+	VHDL2019 =         2019
+
+	__VERSION_MAPPINGS__: Dict[Union[int, str], Enum] = {
+		87:     VHDL87,
+		93:     VHDL93,
+		2:      VHDL2002,
+		8:      VHDL2008,
+		19:     VHDL2019,
+		1987:   VHDL87,
+		1993:   VHDL93,
+		2002:   VHDL2002,
+		2008:   VHDL2008,
+		2019:   VHDL2019,
+		"87":   VHDL87,
+		"93":   VHDL93,
+		"02":   VHDL2002,
+		"08":   VHDL2008,
+		"19":   VHDL2019,
+		"1987": VHDL87,
+		"1993": VHDL93,
+		"2002": VHDL2002,
+		"2008": VHDL2008,
+		"2019": VHDL2019
+	}
+
+	def __init__(self, *_) -> None:
+		"""Patch the embedded MAP dictionary"""
+		for k, v in self.__class__.__VERSION_MAPPINGS__.items():
+			if ((not isinstance(v, self.__class__)) and (v == self.value)):
+				self.__class__.__VERSION_MAPPINGS__[k] = self
+
+	@classmethod
+	def Parse(cls, value: Union[int, str]) -> 'Enum':
+		try:
+			return cls.__VERSION_MAPPINGS__[value]
+		except KeyError:
+			ValueError("Value '{0!s}' cannot be parsed to member of {1}.".format(value, cls.__name__))
+
+	def __lt__(self, other: Enum) -> bool:
+		return self.value < other.value
+
+	def __le__(self, other: Enum) -> bool:
+		return self.value <= other.value
+
+	def __gt__(self, other: Enum) -> bool:
+		return self.value > other.value
+
+	def __ge__(self, other: Enum) -> bool:
+		return self.value >= other.value
+
+	def __ne__(self, other: Enum) -> bool:
+		return self.value != other.value
+
+	def __eq__(self, other: Enum) -> bool:
+		if ((self is self.__class__.Any) or (other is self.__class__.Any)):
+			return True
+		else:
+			return (self.value == other.value)
+
+
+	def __str__(self) -> str:
+		return "VHDL'" + str(self.value)[-2:]
+
+	def __repr__(self) -> str:
+		return str(self.value)
 
 
 @export
@@ -256,24 +377,58 @@ class LabeledEntity:
 		"""Returns a model entity's label."""
 		return self._label
 
+@export
+class MixinDesignUnitWithContext:
+	_contextItems:      Nullable[List['ContextUnion']]
+	_libraryReferences: Nullable[List['LibraryClause']]
+	_packageReferences: Nullable[List['UseClause']]
+	_contextReferences: Nullable[List['ContextReference']]
+
+	def __init__(self, contextItems: Iterable['ContextUnion'] = None):
+		from pyVHDLModel.SyntaxModel import LibraryClause, UseClause, ContextReference
+
+		if contextItems is not None:
+			self._contextItems = []
+			self._libraryReferences = []
+			self._packageReferences = []
+			self._contextReferences = []
+
+			for item in contextItems:
+				self._contextItems.append(item)
+				if isinstance(item, UseClause):
+					self._packageReferences.append(item)
+				elif isinstance(item, LibraryClause):
+					self._libraryReferences.append(item)
+				elif isinstance(item, ContextReference):
+					self._contextReferences.append(item)
+
+	@property
+	def ContextItems(self) -> List['ContextUnion']:
+		return self._contextItems
+
+	@property
+	def LibraryReferences(self) -> List['LibraryClause']:
+		return self._libraryReferences
+
+	@property
+	def PackageReferences(self) -> List['UseClause']:
+		return self._packageReferences
+
+	@property
+	def ContextReferences(self) -> List['ContextReference']:
+		return self._contextReferences
+
 
 @export
 class DesignUnit(ModelEntity, NamedEntity):
 	"""
 	A ``DesignUnit`` is a base-class for all design units.
 	"""
-	_contextItems: List['Context']
 
-	def __init__(self, identifier: str, contextItems: Iterable['Context'] = None):
+	def __init__(self, identifier: str):
 		super().__init__()
 		NamedEntity.__init__(self, identifier)
 
-		self._contextItems = [] if contextItems is None else [c for c in contextItems]
-
-	@property
-	def ContextItems(self) -> List['Context']:
-		return self._contextItems
-
 
 @export
 class PrimaryUnit(DesignUnit):
